/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.0, (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.erlang.org/EPL1_0.txt
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Erlang-4.7.3, December, 1998.
 * 
 * The Initial Developer of the Original Code is Ericsson Telecom
 * AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
 * Telecom AB. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.''
 */
/* Created:  26 Jul 1995 by tobbe@erix.ericsson.se
 *
 * Function: Example of a C client, connecting itself, and
 *           making calls to Erlang.
 *
 * RCS: $Id: igsock_client.c,v 1.2 1996/04/09 13:03:07 tobbe Exp tobbe $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#ifndef __WIN32__
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#else
#include <winsock2.h>
#include <io.h>
#endif __WIN32__
#include <sys/time.h>
#include "etime.h"

#ifdef SUNOS4
extern int fprintf();
extern void perror();
extern int select();
#endif
extern int close();

extern int igsock_connect();
extern etime get_etime(void);

extern int ig_errno;

#define MY_TIMEOUT 1

static void syserr(char *msg) 
{ 
  perror(msg); 
  exit(-1);
} /* syserr */

static void sleep(int sec)
{
  struct timeval tv;

  tv.tv_sec = sec;
  tv.tv_usec = 0;

  if (select(0,(fd_set *)0,(fd_set *)0,(fd_set *)0,(struct timeval *) &tv)<0)
    syserr("select");

} /* sleep */


int main(int argc, char *argv[])
{
  char *host;
  int port,fd;
  int connected=0;
  etime et;

  if (argc == 3) {
    host = argv[argc-2];
    port = atoi(argv[argc-1]);
  }
  else {
    fprintf(stderr,"Usuage: %s <Host> <PortNo>\n",argv[0]);
    exit(-1);
  }

  erl_init((Erl_Heap *) NULL, 0);
  while (1) {

    sleep( MY_TIMEOUT );

    if (!connected) { 
      if ((fd = igsock_connect( host , port )) > 0) {
	connected = 1;
	et = get_etime();
	if (!ig_errno)
	  fprintf(stderr,"Erlang time is %d:%d:%d !\n",
		  et.hour,et.minute,et.second);
	else {
	  fprintf(stderr,"Erlang call failed !\n");
	  close(fd);
	  exit(-1);
	}
      }
      else
	fprintf(stderr,"No connection !\n");
    }
    else {
      et = get_etime();
      if (!ig_errno)
	fprintf(stderr,"Erlang time is %d:%d:%d !\n",
		et.hour,et.minute,et.second);
      else {
	fprintf(stderr,"Erlang call failed !\n");
	close(fd);
	exit(-1);
      }
    }

  } /* while */

  fprintf(stderr,"Ready !\n");

}


