/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.0, (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.erlang.org/EPL1_0.txt
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Erlang-4.7.3, December, 1998.
 * 
 * The Initial Developer of the Original Code is Ericsson Telecom
 * AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
 * Telecom AB. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.''
 */
/* Created:  31 Jul 1995 by tobbe@erix.ericsson.se
 *
 * Function: Example of an autonomous C server, both accepting 
 *           connection attempts from Erlang, and trying to 
 *           connect to Erlang. It is kind of fault tolerant...
 *
 * RCS: $Id: igsock_duplex.c,v 1.2 1996/04/09 13:03:27 tobbe Exp tobbe $
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include "duplex.h"

#ifdef SUNOS4
extern int fprintf();
extern void perror();
extern int select();
extern void bzero();
#endif

#ifdef SOLARIS
extern void *memset();
#endif

extern int close();

extern int igsock_connect();
extern int igsock_listen();
extern int igsock_accept();
extern int ig_errno;
extern int ig_call(void);
extern etime get_etime(void);

#define SHORT_TIMEOUT 1
#define LONG_TIMEOUT 5

static void syserr(char *msg) 
{ 
  perror(msg); 
  exit(-1);
} /* syserr */

int main(int argc, char *argv[])
{
  char *host;
  int port,eport,rc,fd,connected=0;
  int ifd=IG_NO_FD,ofd=IG_NO_FD,lfd=IG_NO_FD;
  struct timeval tv,*tvp;
  fd_set readmask;
  etime et;

  if (argc == 4) {
    host = argv[argc-3];
    port = atoi(argv[argc-2]);
    eport = atoi(argv[argc-1]);
  }
  else {
    fprintf(stderr,"Usuage: %s <Host> <ListenPortNo> <ErlangPortNo\n",argv[0]);
    exit(-1);
  }

  erl_init((Erl_Heap *) NULL, 0);

  if ((lfd = igsock_listen( port )) < 0) {
    perror("igsock_listen: ");
    exit(-1);
  }

  tv.tv_sec = LONG_TIMEOUT;
  tv.tv_usec = 0;
  tvp = &tv;

  if ((ofd = igsock_connect( host , eport )) > 0) { 
    connected = 1; 
    tvp->tv_sec = SHORT_TIMEOUT; 
    et = get_etime(); 
    if (!ig_errno) 
      fprintf(stderr,"Erlang time is %d:%d:%d !\n",et.hour,et.minute,et.second); 
    else { 
      fprintf(stderr,"Erlang call failed !\n"); 
      connected = 0; 
      close(ofd); 
      tvp->tv_sec = LONG_TIMEOUT; 
    } 
  } 
  else 
    fprintf(stderr,"No connection to Erlang !\n");

  /* Ok, in the initial state we will try to setup a listen socket 
   * making it possible for Erlang to connect to us. We will also 
   * try to connect ourselves to Erlang, and if the connect fails 
   * then retry every LONG_TIMEOUT. When the connect is successful 
   * we will get the "Erlang time" every SHORT_TIMEOUT. When Erlang 
   * tries to connect itself to us we will try to accept the 
   * connection and close the listen socket. When something
   * goes wrong we will try to get back to the initial state.
   */
  while (1) {

    FD_ZERO( &readmask );

    if (lfd != IG_NO_FD) 
      FD_SET( lfd , &readmask );     /* Waiting for a remote connect */ 

    if (ifd != IG_NO_FD) 
      FD_SET( ifd , &readmask );     /* Waiting for Erlang requests */ 

    if ((rc = select (FD_SETSIZE, &readmask, (fd_set *)0, (fd_set *)0,
                      (struct timeval *) tvp  )) < 0 ) 
      syserr("select");

    /* TIMEOUT: if not connected try to connect, and in case 
     * of success then get the Erlang time every timeout from now.
     */
    if ((rc == 0) && !connected) {
      if ((ofd = igsock_connect( host , eport )) > 0) { 
	connected = 1; 
	tvp->tv_sec = SHORT_TIMEOUT; 
      } 
      else 
	fprintf(stderr,"No connection to Erlang !\n");
    }
    else if ((rc == 0) && connected) {
      et = get_etime();
      if (!ig_errno)
	fprintf(stderr,"Erlang time is %d:%d:%d !\n",et.hour,et.minute,et.second);
      else {
	fprintf(stderr,"Erlang call failed, errno = %d !\n",ig_errno); 
	connected =0;
	close(ofd);
	tvp->tv_sec = LONG_TIMEOUT;
      }
    }

    /* Connect attempt from Erlang. Try to accept the attempt, and
     * in case of success then also try to connect to Erlang.
     */ 
    if ((lfd != IG_NO_FD) && FD_ISSET( lfd , &readmask )) { 
      FD_CLR( lfd , &readmask );
      if ((fd = igsock_accept( lfd )) < 0) 
	perror("Accept failed: ");
      else {
	ifd = fd;
	close( lfd );
	lfd = IG_NO_FD;
	/* Since Erlang seems to be awake, then try to connect ourselves */
	if (!connected && ((ofd = igsock_connect( host , eport )) > 0)) { 
	  connected = 1; 
	  tvp->tv_sec = SHORT_TIMEOUT; 
	}
      }
    } 

    /* Call from Erlang
     */
    if ((ifd != IG_NO_FD) && FD_ISSET( ifd , &readmask )) { 
      FD_CLR( ifd , &readmask );
      fprintf(stderr,"Erlang call !\n");
      if (ig_call() != IG_OK) {
	fprintf(stderr,"Connection broken, restarting listen socket... !\n");
	close(ifd);
	ifd = IG_NO_FD;
	if ((lfd = igsock_listen( port )) < 0) {
	  perror("igsock_listen (3): ");
	  exit(-1);
	}
      }
    }

  }/* while */

  fprintf(stderr,"Ready !\n");

}


