%% Copyright (C) 1994,1995, Ellemtel Telecommunications Systems Laboratories
%%-----------------------------------------------------------------------
%% Created:  94-10-27 by Tobbe (tobbe@erix.ericsson.se)
%% 
%% Function: Parser for a tiny subset of C. The intended use
%%           is for the Interface Generator.
%%
%%-----------------------------------------------------------------------
%% RCS: $Id: igparser.yrl,v 1.10 1996/04/10 13:42:58 tobbe Exp $

Nonterminals root statement statements typedef_stmt prototype_stmt
	     member_list member_declaration param_decl param_decls
	     std_type_name std_opt_identifier ig_type_name_0 opt_tag
	     constant_expression additive_expression multiplicative_expression 
	     unary_expression constant member_name 
	     param_declarations struct_stmt string hdef_const
	     hdef_stmt other_constants struct_param var_stmt opt_identifier
	     type_qualifier std_declaration	std_qualifiers 
	     std_qualifier storage_class type_qualifiers type_qualifier
	     std_type_spec ig_type_name std_identifier declaration ig_keyword
	     std_typedefname opt_identifier std_type_name_0 tag
	     std_type_spec_0 opt_storage_class assumed_pointer .

Terminals typedef struct int 'IG_string' char 'IG_binaryPtr' long
	  short 'IG_void' void unsigned '*' ',' ';' ')' '(' '{' '}'
	  identifier typedefname 'IG_call' 'IG_cast' 'IG_extern'
	  integerconstant floatingconstant signed octalconstant
	  hexconstant characterconstant '[' ']' '+' '-' '/' '%'
	  stringconstant 'IG_define' 'IG_var' static extern register
	  auto 'IG_fun' const volatile float double 'IG_ifdef'
	  'IG_ifndef' 'IG_else' 'IG_endif' define ifdef ifndef else
	  endif ig_preproc .

Rootsymbol root.

Endsymbol '$end'.

root -> statements : '$1'.

statements ->
	'$empty' : [].
statements ->
	statements statement 
	: lists:append('$1',['$2']).

statement -> 
	typedef_stmt : '$1'.
statement -> 
	struct_stmt : '$1'.
statement -> 
	hdef_stmt : '$1'.
statement -> 
	var_stmt : '$1'.
statement -> 
	prototype_stmt : '$1'.

%%--- V A R I A B L E S ---

%% Returns: {ig_var,Line,Identifier,Declaration}

var_stmt -> 'IG_var' declaration std_identifier ';'	
	: {ig_var, line('$4'), '$3' , mk_decl('$2') }.
%% Ok, this is really a temporary solution. The correct way
%% would be for 'declaration' to handle this.
var_stmt -> 'IG_var' opt_storage_class struct tag std_identifier ';'
	: D = mk_decl(mk_type(type,mk_struct(name('$4')))),
	  Ds= D#declarations.ds,
	  {ig_var, line('$6'), '$5' , #declarations{ds=lists:append(['$2',Ds])} }.
var_stmt -> 'IG_var' identifier std_identifier ';'
	: {ig_var, line('$4'), '$3', mk_decl(mk_type(assumed_ptr,name('$2'))) }.

opt_storage_class -> '$empty'		: [].
opt_storage_class -> storage_class	: '$1'.


%%--- H A S H  D E F I N E S ---

%% Returns: {DefKeword,Line,Identifier,Constant}

%% Added by PL 970204
hdef_stmt -> ig_preproc define identifier hdef_const
	: {hdef, line('$1'), mk_identifier(id,name('$3'),[]) , '$4'}.
hdef_stmt -> ig_preproc define identifier
	: {hdef, line('$1'), mk_identifier(id,name('$3'),[]) , 
			{identifier,line('$1'),true}}.
hdef_stmt -> ig_preproc ifdef identifier
	: {ifdef, line('$1'), mk_identifier(id,name('$3'),[]) }.
hdef_stmt -> ig_preproc ifndef identifier	
	: {ifndef, line('$1'), mk_identifier(id,name('$3'),[]) }.
hdef_stmt -> ig_preproc else	: {else, line('$1')}.
hdef_stmt -> ig_preproc endif	: {endif, line('$1')}.

%% Old pre-proc stuff, kept for backw. compat.
hdef_stmt -> 'IG_define' identifier hdef_const
	: {hdef, line('$3'), mk_identifier(id,name('$2'),[]) , '$3'}.
hdef_stmt -> 'IG_define' identifier
	: {hdef, line('$2'), mk_identifier(id,name('$2'),[]) , {identifier,line('$2'),true}}.
hdef_stmt -> 'IG_ifdef' identifier	
	: {ifdef, line('$2'), mk_identifier(id,name('$2'),[]) }.
hdef_stmt -> 'IG_ifndef' identifier	
	: {ifndef, line('$2'), mk_identifier(id,name('$2'),[]) }.
hdef_stmt -> 'IG_else'	: {else, line('$1')}.
hdef_stmt -> 'IG_endif'	: {endif, line('$1')}.

hdef_const -> octalconstant	: '$1'. 
hdef_const -> hexconstant	: '$1'. 
hdef_const -> other_constants	: '$1'. 
hdef_const -> string		: '$1'. 
hdef_const -> identifier	: '$1'.
%	: mk_hdef(lists:concat(["?",name('$1')]),line('$1')). %% A previously defined macro ?

%%--- T Y P E D E F ---

%% Returns: {type,Line,Identifier,Members}
typedef_stmt ->	typedef struct opt_tag '{' member_list '}' std_typedefname';'
	: {type, line('$8'), '$7', '$5'}.
%% Returns: {type,Line,Identifier,Declaration}
typedef_stmt ->	typedef std_type_spec std_typedefname ';'
	: {type, line('$4'), '$3', mk_decl('$2')}.

%%--- S T R U C T ---

%% Returns: {type,Line,Identifier,Members}
struct_stmt ->
	struct identifier '{' member_list '}' ';'
	: {type, line('$6'), mk_identifier(id,mk_struct(name('$2')),[]) , '$4'}.

member_list -> member_declaration 		: '$1'.
member_list -> member_list member_declaration	: lists:append('$1','$2').

%% A member is represented as: {Declaration,Line,MemberIdentifier}
%% where MemberIdentifier may be an Identifier or an AggregateIdentifier
member_declaration ->
	std_type_spec member_name 
	: [mk_member(mk_decl('$1'), line('$2'), name('$2'))].
member_declaration ->
	'IG_binaryPtr' std_identifier ';'
	: [mk_member(mk_decl(binaryPtr), line('$3') , '$2')].
member_declaration ->
	identifier std_identifier ';'
	: [mk_member(mk_decl(mk_type(assumed_ptr,name('$1'))), line('$2'), name('$2'))].

member_name -> std_identifier ';' 				: mk_token('$1',line('$2')).
member_name -> std_identifier '[' constant_expression ']' ';'	: mk_token(mk_aggregate('$1', '$3'), line('$5')).

%%--- C O N S T A N T  E X P R E S S I O N ---

%% We don't use the line number information since
%% the evaluation takes places here, at parse time !!

constant_expression ->
	additive_expression
	: constant_eval('$1').

additive_expression ->
        multiplicative_expression 
	: '$1'.
additive_expression -> 
	additive_expression '+' multiplicative_expression 
	: {name('$2'), '$1', '$3'}.
additive_expression ->  
	additive_expression '-' multiplicative_expression 
	: {name('$2'), '$1', '$3'}.

multiplicative_expression ->
        unary_expression 
	: '$1'.
multiplicative_expression -> 
	multiplicative_expression '*' unary_expression 
	: {name('$2'), '$1', '$3'}.
multiplicative_expression ->  
	multiplicative_expression '/' unary_expression 
	: {name('$2'), '$1', '$3'}.
multiplicative_expression ->   
	multiplicative_expression '%' unary_expression 	
	: {name('$2'), '$1', '$3'}.
multiplicative_expression ->   
	multiplicative_expression '(' additive_expression ')'
	: {name('$2'), '$1', '$3'}.

unary_expression -> octalconstant 		: base_convert(name('$1'),8).
unary_expression -> hexconstant 		: base_convert(name('$1'),16).
unary_expression -> other_constants   		: name('$1').
unary_expression -> '(' constant_expression ')'	: '$2'.

other_constants -> integerconstant 	: '$1'.
other_constants -> floatingconstant 	: '$1'.
other_constants -> characterconstant 	: '$1'.


%%--- F U N C T I O N  P R O T O T Y P E S ---

%% Returns: {IgKeyword,Line,Identifier,Declaration,Parameters}
%%      or: {IgKeyword,Line,Identifier,Parameters}

prototype_stmt ->
	ig_keyword declaration std_identifier '(' param_declarations ')' ';'
	: {'$1', line('$7'), '$3', mk_decl('$2'), '$5'}.
prototype_stmt ->
	ig_keyword struct identifier std_identifier '(' param_declarations ')' ';'
	: {'$1', line('$8'), '$4', mk_decl(mk_type(type,mk_struct(name('$3')))) , '$6'}.
prototype_stmt ->
	ig_keyword 'IG_binaryPtr' identifier '(' param_declarations ')' ';'
	: {'$1', line('$7'), mk_identifier(id,name('$3'),[]) , mk_decl(binaryPtr) , '$5'}.
prototype_stmt ->
	ig_keyword identifier std_identifier '(' param_declarations ')' ';'
	: {'$1', line('$7'), '$3', mk_decl(mk_type(assumed_ptr,name('$2'))) , '$5'}.

ig_keyword -> 'IG_fun' 		: ig_fun.
ig_keyword -> 'IG_call' 	: ig_call.
ig_keyword -> 'IG_extern'	: ig_extern.
ig_keyword -> 'IG_void' 	: ig_void.
ig_keyword -> 'IG_cast' 	: ig_cast.

param_declarations -> param_decls	: '$1'.

%% We don't handle a terminating ellipsis !
param_decls -> param_decl 			: '$1'.
param_decls -> param_decls ',' param_decl 	: lists:append('$1','$3').

%% A parameter is represented as: {Declaration,Identifier} 
%% or as: Declaration
param_decl -> std_type_spec std_opt_identifier	: [mk_param(mk_decl('$1'),'$2')].
param_decl -> identifier std_opt_identifier 		: [mk_param(mk_decl(mk_type(assumed_ptr,name('$1'))),'$2')].
param_decl -> 'IG_binaryPtr' std_opt_identifier		: [mk_param(mk_decl(binaryPtr),'$2')].
param_decl -> struct struct_param 			: '$2'.

struct_param ->	identifier std_opt_identifier 		
	: [mk_param(mk_decl(mk_type(type, mk_struct(name('$1')))),'$2')].

%%--- C O M M O N  S T U F F ---

std_typedefname -> typedefname				: mk_identifier(type,name('$1'),[]).
std_typedefname -> '*' typedefname			: mk_identifier(type_ptr,name('$2'),[]).
std_typedefname -> '*' type_qualifiers typedefname	: mk_identifier(type_ptr,name('$3'),'$2').

std_identifier -> identifier				: mk_identifier(id,name('$1'),[]).
std_identifier -> '*' identifier			: mk_identifier(ptr,name('$2'),[]).
std_identifier -> '*' type_qualifiers identifier	: mk_identifier(ptr,name('$3'),'$2').

std_opt_identifier -> opt_identifier			: mk_identifier(id,name('$1'),[]).
std_opt_identifier -> '*' opt_identifier		: mk_identifier(ptr,name('$2'),[]).
std_opt_identifier -> '*' type_qualifiers opt_identifier: mk_identifier(ptr,name('$3'),'$2').

opt_identifier -> '$empty'	: [].
opt_identifier -> identifier	: '$1'.

declaration -> std_qualifiers	: '$1'.
declaration -> type_qualifiers	: '$1'.
declaration -> std_type_spec	: '$1'.
declaration -> std_declaration	: '$1'.

std_declaration -> std_qualifiers ig_type_name		: lists:append('$1','$2').
std_declaration -> std_type_spec storage_class	 	: lists:append('$1','$2').
std_declaration -> std_declaration std_qualifier 	: lists:append('$1','$2').
std_declaration -> std_declaration ig_type_name 	: lists:append('$1','$2').

assumed_pointer -> identifier	: [mk_type(assumed_ptr,name('$1'))].

std_type_spec -> ig_type_name			: '$1'.
std_type_spec -> type_qualifiers ig_type_name	: lists:append('$1','$2').
std_type_spec -> std_type_spec type_qualifier	: lists:append('$1','$2').
std_type_spec -> std_type_spec ig_type_name	: lists:append('$1','$2').

std_type_spec_0 -> ig_type_name_0			: '$1'.
std_type_spec_0 -> type_qualifiers ig_type_name_0	: lists:append('$1','$2').
std_type_spec_0 -> std_type_spec_0 type_qualifier	: lists:append('$1','$2').
std_type_spec_0 -> std_type_spec_0 ig_type_name_0	: lists:append('$1','$2').

std_qualifiers -> storage_class			: '$1'.
std_qualifiers -> type_qualifiers storage_class	: lists:append('$1','$2').
std_qualifiers -> std_qualifiers std_qualifier	: lists:append('$1','$2').

std_qualifier -> storage_class	: '$1'.
std_qualifier -> type_qualifier	: '$1'.

type_qualifiers -> type_qualifier			: '$1'.
type_qualifiers -> type_qualifiers type_qualifier	: lists:append('$1','$2').

type_qualifier -> const		: [name('$1')].
type_qualifier -> volatile	: [name('$1')].

storage_class -> static 	: [name('$1')].
storage_class -> extern 	: [name('$1')].
storage_class -> auto 		: [name('$1')].
storage_class -> register	: [name('$1')].
%% storage_class -> typedef	: [name('$1')].  Left it out on purpose !!

std_type_name -> std_type_name_0	: '$1'.
std_type_name -> void			: [name('$1')].

std_type_name_0 -> int 		: [name('$1')].
std_type_name_0 -> char		: [name('$1')].
std_type_name_0 -> short	: [name('$1')].
std_type_name_0 -> long	 	: [name('$1')].
std_type_name_0 -> signed 	: [name('$1')].
std_type_name_0 -> unsigned 	: [name('$1')].
std_type_name_0 -> float 	: [name('$1')].
std_type_name_0 -> double 	: [name('$1')].

ig_type_name -> 'IG_string'	: [name('$1')].  
ig_type_name -> typedefname	: [mk_type(type,name('$1'))].  
ig_type_name -> std_type_name	: '$1'. 

ig_type_name_0 -> 'IG_string'		: [name('$1')].  
ig_type_name_0 -> typedefname		: [mk_type(type,name('$1'))].  
ig_type_name_0 -> std_type_name_0	: '$1'. 

constant -> integerconstant 	: '$1'.
constant -> floatingconstant 	: '$1'.
constant -> octalconstant 	: '$1'.
constant -> hexconstant 	: '$1'.
constant -> characterconstant 	: '$1'.

string -> stringconstant	: '$1'.

opt_tag -> '$empty'	: [].
opt_tag -> tag		: '$1'.

tag -> identifier	: '$1'.
tag -> typedefname	: '$1'.

Erlang code.

-include("igabsform.hrl").

%% Evaluate a constant expression. At failure, throw an error.
constant_eval(N) when number(N) -> 
	N;
constant_eval({'+',N1,N2}) when number(N1),number(N2) ->
	N1 + N2;
constant_eval({'-',N1,N2}) when number(N1),number(N2) ->
	N1 - N2;
constant_eval({'*',N1,N2}) when number(N1),number(N2) ->
	N1 * N2;
constant_eval({'/',N1,N2}) when number(N1),number(N2) ->
	N1 / N2;
constant_eval({'%',N1,N2}) when number(N1),number(N2) ->
	N1 rem N2;
constant_eval({Op,T,X}) when tuple(T) ->
	constant_eval({Op,constant_eval(T),X});
constant_eval({Op,X,T}) when tuple(T) ->
	constant_eval({Op,X,constant_eval(T)});
constant_eval(XX) ->
	io:format("~nigparser: constant_eval got: ~w~n",[XX]),
	throw(error).

%% Strip off the Name of a token (if any)
name({Name,Line}) -> Name;
name({Cat,Line,Name}) -> Name; 
name(X) -> X.

%% Strip off the Line number of a token (if any)
line({Name,Line}) -> Line;
line({Cat,Line,Name}) -> Line;
line(X) -> X.

base_convert({Suff,L},Base) ->
	{Suff,base(L, Base, 0)};
base_convert(L,Base) ->
	base(L, Base, 0).

base([C|Cs], Base, SoFar) when C >= $0, C =< $9, C < Base + $0 ->
	Next = SoFar * Base + (C - $0),
	base(Cs, Base, Next);
base([C|Cs], Base, SoFar) when C >= $a, C =< $f, C < Base + $a - 10 ->
	Next = SoFar * Base + (C - $a + 10),
	base(Cs, Base, Next);
base([C|Cs], Base, SoFar) when C >= $A, C =< $F, C < Base + $A - 10 ->
	Next = SoFar * Base + (C - $A + 10),
	base(Cs, Base, Next);
base([], _, N) ->
	N.

%% Constructor functions
 
mk_struct(X) -> list_to_atom(lists:concat(["struct ",X])).
mk_param(Decl,Id) -> {Decl,Id}.
mk_token(X,Y) -> {X,Y}.
mk_aggregate(X,Y) -> {X,Y}.
mk_type(X,Y) -> {X,Y}.
mk_member(Decl,Line,MemId) -> {Decl,Line,MemId}.

%% NOTE: We assume that the igparser.hrl includes the proper
%% definitions of the records for the abstract forms.

mk_decl(X) when list(X) -> #declarations{ds=X};	
mk_decl(X) -> #declarations{ds=[X]}.
mk_identifier(Cat,Name,Tq) -> #identifier{cat=Cat,name=Name,tq=Tq}.



