/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.0, (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.erlang.org/EPL1_0.txt
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Erlang-4.7.3, December, 1998.
 * 
 * The Initial Developer of the Original Code is Ericsson Telecom
 * AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
 * Telecom AB. All Rights Reserved.
 * 
 * Contributor(s): ______________________________________.''
 */
#ifndef _IG_H
#define _IG_H
/*--- Copyright (C) 1994,1995 Ellemtel Telecommunications Systems Laboratories
 *---
 *--- This is the includefile for the Erlang
 *--- Interface Generator (IG) Module
 *--- Created: 31 Oct 1994 by tobbe@erix.ericsson.se
 *---
 *--- RCS: $Id: ig.h,v 1.10 1996/04/10 06:24:28 tobbe Exp $
 */
#include "erl_interface.h"

#define IG_OK 0                /* success */
#define IG_READ_ERROR -1       /* if a ig_read fails */
#define IG_WRITE_ERROR -2      /* if a ig_write fails */
#define IG_DECODE_ERROR -3     /* if the term decoding fails */
#define IG_ENCODE_ERROR -4     /* if the term encoding overwrites its buffer */
#define IG_NO_FD -1            /* not a valid filedescriptor */

/*--- Function pointer to be used for defining
 *--- a (possible) user init routine.
 */
typedef int (*IG_PTR)();

/*--- The type IG_string has a special meaning.
 *--- Strings will be passed between the Erlang
 *--- side and the C side, i.e they can be used
 *--- as arguments or return values.
 *--- 7 Aug 1995: changed from typedef to #define,
 *--- because of conflicts when generating extern
 *--- declarations.
 */
#define IG_string char *

/*--- The type IG_binaryPtr has a special meaning.
 *--- Binaries will pass a chunk of unsigned memory
 *--- between the Erlang side and the C side, i.e they 
 *--- can be used both as arguments or return values.
 *--- The IG_binary struct can be used to build such
 *--- a binary.
 */
typedef struct {
  int size;
  unsigned char *binp;
} IG_binary,*IG_binaryPtr;

/*--- Use IG_fun as the first identifier in a function
 *--- prototype. This will indicate that the following
 *--- function shall be callable from Erlang.
 */
#define IG_fun 
#define IG_fun_tag 0

/*--- Use IG_call as the first identifier in a function
 *--- prototype. This will generate a C function that sends
 *--- its arguments to the Erlang side, where the corresponding
 *--- function is called. It will then wait for the result to 
 *--- be returned.
 */
#define IG_call 
#define IG_call_tag 1

/*--- Use IG_cast as the first identifier in a function
 *--- prototype. This will generate a C function that sends
 *--- its arguments to the Erlang side, where the corresponding
 *--- function is called. 
 */
#define IG_cast 
#define IG_cast_tag 2

/*--- Use IG_extern as the first identifier in a function
 *--- prototype. This will make it possible to get the
 *--- address of the function from the Erlang side.
 */
#define IG_extern extern
#define IG_extern_tag 3

/*--- Use IG_void as the first identifier in a function
 *--- prototype. This will make the function call asyncronous,
 *--- i.e the result will be discarded.
 */
#define IG_void 
#define IG_void_tag 4

/*--- Use these IG preprocessor commands if you want to generate
 *--- the corresponding Erlang things into a .hrl file.
 *--- NOTE: The new comment style syntax SHOULD be used instead.
 *--- see the user's guide for reference.
 */
#ifdef __GNUC__
#ifndef DONT_USE_CPP
#define IG_define #define
#define IG_ifdef #ifdef
#define IG_ifndef #ifndef
#define IG_else #else
#define IG_endif #endif
#endif
#endif

/*--- Use IG_var for variables that shall be accessible from Erlang
 *--- NOTE: The new comment style syntax SHOULD be used instead.
 *--- see the user's guide for reference.
 */
#ifdef __GNUC__
#define IG_var #define IG_ignore
#endif
#define IG_var_tag 5

/*--- For internal use when to get the size of a variable
 */
#define IG_size_tag 6

/*--- All CSASL hooks are declared here to get the AbsForm from the
 *--- parser right. The hooks here will never be seen in the user 
 *--- code because the macro CSASL_HOOKS_NEVER_DEFINED must never 
 *--- be defined
 */
#ifdef CSASL_HOOKS_NEVER_DEFINED
IG_fun int erl_c_init(IG_binaryPtr);
IG_fun int erl_c_terminate(void);
IG_fun int erl_c_suspend(void);
#endif

#endif

